#include <vector>
#include "Contour.h"

using namespace std;


double contourLaplacian(IplImage *image, int x, int y) {
    return iPixel( image, x-1, y) + iPixel( image, x+1, y) +
           iPixel( image, x, y-1) + iPixel( image, x, y+1) +
                 (-4.0 * iPixel( image, x, y) );
}

Vector2D contourLaplacianDifference(IplImage *image, int x, int y) {
    return Vector2D( contourLaplacian( image, x + 1, y ) - contourLaplacian( image, x - 1, y ),
                     contourLaplacian( image, x, y + 1 ) - contourLaplacian( image, x, y - 1 ) );
}

Vector2D contourIsotopeGradient(IplImage *image, int x, int y) {
    double dx = 0.0, dy = 0.0, d = 0.0;
    int r = 1;
    while( d == 0.0 ) {
        double k = (1.0 / (2.0 * (double)(r)));
        double cdx = iPixel(image, x + r, y) - iPixel(image, x - r, y);
        double cdy = iPixel(image, x, y + r) - iPixel(image, x, y - r);
        dx = k * cdx; dy = k * cdy;
        d = sqrt(iSq(dx)+iSq(dy));
        r++;
        if( r > CONTOUR_GRADIENT_MAX_RADIUS ) return Vector2D(0.0, 0.0);
    }
    return Vector2D( - dy , dx );
}

double contourSlopeLimitedNorm(IplImage *image, int x, int y, double bn) {
    double delta = 0.0;
    if(bn > 0.0) {
        delta = sqrt( iSq(iMinZero( -iPixel(image, x-1, y) + iPixel(image, x, y) ) ) +
                      iSq(iMaxZero( +iPixel(image, x+1, y) - iPixel(image, x, y) ) ) +
                      iSq(iMinZero( -iPixel(image, x, y-1) + iPixel(image, x, y) ) ) +
                      iSq(iMaxZero( +iPixel(image, x, y+1) - iPixel(image, x, y) ) ) );
    } else if(bn < 0.0) {
        delta = sqrt( iSq(iMaxZero( -iPixel(image, x-1, y) + iPixel(image, x, y) ) ) +
                      iSq(iMinZero( +iPixel(image, x+1, y) - iPixel(image, x, y) ) ) +
                      iSq(iMaxZero( -iPixel(image, x, y-1) + iPixel(image, x, y) ) ) +
                      iSq(iMinZero( +iPixel(image, x, y+1) - iPixel(image, x, y) ) ) );
    }
    return delta;
}



void contourCopyFloat( IplImage *src, IplImage *dst ) {
    for(int a = 0; a < src->height; a++) {
        for(int b = 0; b < src->width; b++) {
            CvScalar p = cvGet2D( src, a, b );
            cvSet2D( dst, a, b, p );
        }
    }
}

void contourClampNegative( double& v ) {
    if( v < -255.0 ) v = -255.0;
    if( v > 255.0 ) v = 255.0;
}

void contourClamp( double& v ) {
    if( v < 0.0 ) v = 0.0;
    if( v > 255.0 ) v = 255.0;
}

void contourInpaint( IplImage *image, IplImage *mask, IplImage *dst, double dT, int iter) {
    IplImage *ci = iCloneFloat( image, 1.0 );
    IplImage *ci2 = iCloneFloat( image, 1.0 );

    vector< pair<int, int> > maskPixels;
    for(int a = 0; a < ci->height; a++) {
        for(int b = 0; b < ci->width; b++) {
            if( iPixel(mask, a, b) < 128 ) continue;
            maskPixels.push_back(make_pair(a, b));
        }
    }
    
    for(int i = 0; i < iter; i++) {

        if(i % 100 == 0 ) {
            if(processEvents()) return;
        }

        for(int j = 0; j < maskPixels.size(); j++) {

            int a = maskPixels[j].first;
            int b = maskPixels[j].second;

            if( iPixel(mask, a, b) < 128 ) continue;
            double pixel = iPixel( ci, a, b );

            Vector2D dlap = contourLaplacianDifference( ci, a, b );
            Vector2D isotope = contourIsotopeGradient( ci, a, b );
            if( isotope.x == 0.0 && isotope.y == 0.0 ) continue;
            isotope.normalize();

            double bn = dlap * isotope;

            double len = contourSlopeLimitedNorm( ci, a, b, bn );
            double it = bn * len;
            contourClampNegative( it );
            double dI = pixel + (dT * it);
            contourClamp( dI );
            iSixel( ci2, a, b, dI );

        }

        IplImage *tc = ci2;
        ci2 = ci;
        ci = tc;
        
    }

    contourCopyFloat( ci, dst );
    cvReleaseImage( &ci );
    cvReleaseImage( &ci2 );
}