#include <map>
#include "Image.h"
#include "Toolbar.h"

using namespace std;

map< string, int > iGlobalNameTable;
int iGlobalSaveCount = 0;

void iShow( const char *fileName, IplImage *image, bool eq, bool save, float scale ) {
    if(!fileName) fileName = "";
    if(!image) return;
    IplImage *scaleImage = cvCreateImage(cvGetSize(image), IPL_DEPTH_8U, image->nChannels);
    IplImage *image2 = cvCloneImage( image );
    if(eq) {
        cvCvtScale( image2, image2, 128.0 );
        cvAddS( image2, cvScalar(128.0), image2 );
        cvCvtScale( image2, scaleImage, 1.0 );
    } else cvCvtScale( image, scaleImage, scale );
    if(save) {
        char fileNameBuffer[512];
        sprintf( fileNameBuffer, "Output/%s_%.4d.jpg", fileName, iGlobalNameTable[fileName] );
        cvSaveImage( fileNameBuffer, scaleImage );
    }
    cvNamedWindow( fileName, 1 );
    cvShowImage( fileName, scaleImage );
    cvWaitKey(1);
    cvReleaseImage( &scaleImage );
    cvReleaseImage( &image2 );
    iGlobalNameTable[fileName]++;
}

void iShow8( const char *fileName, IplImage *image ) {
    cvNamedWindow( fileName, 1 );
    cvShowImage( fileName, image );
    cvWaitKey(0);
}

void iDisplay( const char *windowName, IplImage *image, IplImage *mask, 
               bool showMask, bool save, const char *fileName, bool colorShowMask ) {
    IplImage *frameImage = cvCreateImage( cvGetSize( image ), image->depth, 3 );
    if( showMask ) {
        for(int a = 0; a < image->height; a++) {
            for(int b = 0; b < image->width; b++) {
                CvScalar pixel = cvGet2D( image, a, b );
                double greyScale = pixel.val[0] * IMAGE_LUMIN_B +
                                   pixel.val[1] * IMAGE_LUMIN_G +
                                   pixel.val[2] * IMAGE_LUMIN_R;
                double greyB = greyScale, greyG = greyScale, greyR = greyScale;
                if( colorShowMask ) {
                    greyB = pixel.val[0];
                    greyG = pixel.val[1];
                    greyR = pixel.val[2];
                }
                if( iPixel( mask, a, b ) > 128 ) {
                    greyB = greyB * IMAGEDISPLAY_MASK_TRANSPARENCY + 
                        IMAGEDISPLAY_MASKCOLOR_B * (1.0 - IMAGEDISPLAY_MASK_TRANSPARENCY);
                    greyG = greyR * IMAGEDISPLAY_MASK_TRANSPARENCY + 
                        IMAGEDISPLAY_MASKCOLOR_G * (1.0 - IMAGEDISPLAY_MASK_TRANSPARENCY);
                    greyR = greyG * IMAGEDISPLAY_MASK_TRANSPARENCY + 
                        IMAGEDISPLAY_MASKCOLOR_R * (1.0 - IMAGEDISPLAY_MASK_TRANSPARENCY);
                }
                pixel.val[0] = greyB; pixel.val[1] = greyG; pixel.val[2] = greyR;
                cvSet2D( frameImage, a, b, pixel );
            }
        }
    } else cvScale( image, frameImage );
    cvNamedWindow( windowName, 1 );
    cvShowImage( windowName, frameImage );
    if( windowOutputPath != "" ) {
        cvSaveImage( windowOutputPath.c_str(), frameImage );
    }
    if( windowSaveInterim ) {
        char fileNameBuffer[1024];
        sprintf(fileNameBuffer, "./Output/image%.4d.jpg", iGlobalSaveCount++);
        cvSaveImage( fileNameBuffer, frameImage );
    }
    cvWaitKey(1);
    cvReleaseImage( &frameImage );
}

double iPixel( IplImage *image, int x, int y, int i ) {
    if( x < 0 ) x = 0; if( x >= image->height ) x = image->height - 1;
    if( y < 0 ) y = 0; if( y >= image->width ) y = image->width - 1;
    if( i == -1 && image->nChannels >= 3 ) {
        CvScalar pixel = cvGet2D( image, x, y );
        return pixel.val[0] * IMAGE_LUMIN_B +
               pixel.val[1] * IMAGE_LUMIN_G +
               pixel.val[2] * IMAGE_LUMIN_R;
    }
    return cvGet2D( image, x, y ).val[i];
}

void iSixel( IplImage *image, int x, int y, double value ) {
    if( x < 0 ) x = 0; if( x >= image->height ) x = image->height - 1;
    if( y < 0 ) y = 0; if( y >= image->width ) y = image->width - 1;
    return cvSet2D( image, x, y, cvScalar(value, value, value, value) );
}

double iMinZero( double v ) { return (v < 0) ? (v) : (0); }

double iMaxZero( double v ) { return (v > 0) ? (v) : (0); }

double iSq( double v ) { return (v*v); }

void iZero( IplImage *image ) {
    if(!image) return;
    for(int a = 0; a < image->height; a++) {
        for(int b = 0; b < image->width; b++) {
            cvSet2D( image, a, b, cvScalar( 0.0, 0.0, 0.0, 0.0 ) );
        }
    }
}

void iMaskedCombine( IplImage *src, IplImage *dst, IplImage *mask, bool maskBinary, double factor ) {
    if(!src || !dst || !mask) return;
    for(int a = 0; a < dst->height; a++) {
        for(int b = 0; b < dst->width; b++) {
            CvScalar d = cvGet2D( dst, a, b);
            CvScalar s = cvGet2D( src, a, b);
            CvScalar m = cvGet2D( mask, a, b);
            if( maskBinary ) {
                if( m.val[0] > 128 ) cvSet2D( dst, a, b, s );
                continue;
            }
            double f = (((double)m.val[0]) / 255.0);
            f = f * f * f;
            f *= factor;
            if( factor < 0.0 ) f -= factor;
            if( f > 1.0 ) f = 1.0;
            if( f <= 0.0 ) continue;
            double rf = (1.0 - f); if ( rf < 0.0 ) rf = 0.0;
            for(int c = 0; c < src->nChannels; c++) {
                d.val[c] = (d.val[c] * rf) + (s.val[c] * f);
                if(d.val[c] > 255.0) d.val[c] = 255.0;
                if(d.val[c] < 0) d.val[c] = 0.0;
            }
            cvSet2D( dst, a, b, d );
        }
    }
}

IplImage * iHalf( IplImage *img ) {
    IplImage *halfImg = cvCreateImage(cvSize(img->width * 0.5, img->height * 0.5), 
                                      img->depth, img->nChannels);
    cvPyrDown( img, halfImg );
    cvReleaseImage( &img );
    return halfImg;
}

IplImage* iCloneFloat( IplImage *image, float scale ) {
    IplImage *ni = cvCreateImage( cvGetSize( image ), IPL_DEPTH_32F, image->nChannels);
    for(int a = 0; a < image->height; a++) {
        for(int b = 0; b < image->width; b++) {
            CvScalar p = cvGet2D( image, a, b );
            p.val[0] /= scale; p.val[1] /= scale; 
            p.val[2] /= scale; p.val[3] /= scale;
            cvSet2D( ni, a, b, p );
        }
    }
    return ni;
}