#ifndef _INPAINT_HEADER_INCLUDE_
#define _INPAINT_HEADER_INCLUDE_

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <opencv/cv.h>
#include <opencv/highgui.h>
#include "Image.h"

#define NUM_IMAGE_SEPARATION 3
#define BUFFER_LEN 512

#define GREYSCALE_LUMIN_B 0.114
#define GREYSCALE_LUMIN_G 0.587
#define GREYSCALE_LUMIN_R 0.299

#define DESATURATE_SCALE 0.3333333

#define VECTOR_LENGTH_SCALE 0.57735
#define VECTOR_SIN_SCALE 255.0

#define IMGI_SRC -1
#define IMGI_MASK -2
#define IMGI_RESULT -3

bool processEvents();

typedef enum {
    SEP_GREYSCALE,
    SEP_DESATURATE,
    SEP_RGB,
    SEP_VECTOR,
    NUM_SEPARATION_METHODS,
} enum_separation_methods;

typedef enum {
    INP_NONE,
    INP_GAUSSIANBLUR,
    INP_FDII,
    INP_BERTALMIO,
    INP_TELEA,
    INP_NAVIER,
    INP_EXEMPLAR,
    INP_LOCAL,
    NUM_INPAINT_METHODS,
} enum_inpaint_methods;

class Inpainter {
private:
    IplImage *imageSource;
    IplImage *imageMask;
    IplImage *image[NUM_IMAGE_SEPARATION];
    IplImage *imageResult[NUM_IMAGE_SEPARATION];
    IplImage *imageResultColor;
    
    IplImage *imageResultCombined;

public:
    int methodSeparation;
    int methodInpaint;
    
    int pGaussianOctaves;
    int pFDIIKernel;
    int pFDIIIterations;
    int pBertalmioInpaintSteps;
    int pBertalmioDiffuseSteps;
    double pBertalmioClimbRate;
    double pBertalmioClimbDecay;
    int pTeleaRadius;
    int pNavierRadius;
    int pExmpIterations;
    int pExmpPatchSize;
    int pExmpBoundary;
    int pExmpGPU;

    Inpainter();
    ~Inpainter();
    void reset();
    void load(const char* fileName, const char* maskName = NULL);
    void separate();
    void prepare();
    void inpaint();
    void combine();

    IplImage * operator [] (int index);

    bool RGBCapableMethod();
};

#endif