#include "Mask.h"

HWND hMaskBrushButton;
HWND hMaskEraseButton;
HWND hMaskFillButton;
HWND hMaskSelectButton;
HWND hMaskDilateButton;
HWND hMaskBorderButton;

HWND hLogEdit;

HWND hMaskBrushSizeLabel;
HWND hMaskBrushSizeTrackbar;
HWND hMaskDilateLabel;
HWND hMaskDilateTrackbar;
HWND hMaskBorderLabel;
HWND hMaskBorderTrackbar;


int maskBrushState = BRUSH_BRUSH;
int maskBrushSize = 8;
int maskDilate = 1;
int maskBorder = 2;

bool maskMouseButtonDown = false;

vector<string> logString;

void maskBuildInterface() {
    // Build Window
    hMaskEditWindow = UIForm( "Mask Editor", WINDOW_TITLE, CW_USEDEFAULT, CW_USEDEFAULT, 450, 248, HWND_DESKTOP, 0 );
    // Build Buttons
    int cx = 8;
    hMaskBrushButton  = UIButton( "Brush" , cx, 8, 48, 40, ID_MASK_BRUSH_BUTTON , hMaskEditWindow ); cx += 56;
    hMaskEraseButton  = UIButton( "Erase" , cx, 8, 48, 40, ID_MASK_ERASE_BUTTON , hMaskEditWindow ); cx += 56;
    hMaskFillButton   = UIButton( "Fill"  , cx, 8, 48, 40, ID_MASK_FILL_BUTTON  , hMaskEditWindow ); cx += 56;
    hMaskSelectButton = UIButton( "Select", cx, 8, 48, 40, ID_MASK_SELECT_BUTTON, hMaskEditWindow ); cx += 56;
    hMaskDilateButton = UIButton( "Dilate", cx, 8, 48, 40, ID_MASK_DILATE_BUTTON, hMaskEditWindow ); cx += 56;
    hMaskBorderButton = UIButton( "Border", cx, 8, 48, 40, ID_MASK_BORDER_BUTTON, hMaskEditWindow ); cx += 56;
    // Build Log Window
    hLogEdit = UIEdit( "", 8, 56, 328, 160, ID_MASK_LOG, hMaskEditWindow, true, false );
    // Build Trackbars
    hMaskBrushSizeLabel = UILabel("BrushSize: 00", 344, 8, 102, 16, -1, hMaskEditWindow);
    hMaskBrushSizeTrackbar = UITrackBar(336, 32, 96, 45, ID_MASK_BRUSHSIZE_TRACKBAR, hMaskEditWindow,
                                        1, 256, maskBrushSize, 8);
    UISetTextPrintf( hMaskBrushSizeLabel, "BrushSize: %d", maskBrushSize);
    hMaskDilateLabel = UILabel("Dilate: 00", 344, 80, 102, 16, -1, hMaskEditWindow);
    hMaskDilateTrackbar = UITrackBar(336, 104, 96, 45, ID_MASK_DILATE_TRACKBAR, hMaskEditWindow,
                                        1, 85, maskDilate, 8);
    UISetTextPrintf( hMaskDilateLabel, "Dilate: %d", maskDilate * 3);
    hMaskBorderLabel = UILabel("Border: 00", 344, 152, 102, 16, -1, hMaskEditWindow);
    hMaskBorderTrackbar = UITrackBar(336, 168, 96, 45, ID_MASK_BORDER_TRACKBAR, hMaskEditWindow,
                                        1, 85, maskBorder, 8);
    UISetTextPrintf( hMaskBorderLabel, "Border: %d", maskBorder * 3);
    ShowWindow( hMaskEditWindow, SW_HIDE );
}

void maskUpdateTrackbars() {
    maskBrushSize = UIGetTrackBar( hMaskBrushSizeTrackbar );
    UISetTextPrintf( hMaskBrushSizeLabel, "BrushSize: %d", maskBrushSize);
    maskDilate = UIGetTrackBar( hMaskDilateTrackbar );
    UISetTextPrintf( hMaskDilateLabel, "Dilate: %d", maskDilate * 3);
    maskBorder = UIGetTrackBar( hMaskBorderTrackbar );
    UISetTextPrintf( hMaskBorderLabel, "Border: %d", maskBorder * 3);

}

bool maskHandledEvent(int ID) {
    return (ID >= ID_MASK_MIN && ID <= ID_MASK_MAX);
}

void maskCommandEvent(int ID) {
    switch( ID ) {
        case ID_MASK_BRUSH_BUTTON:
            maskBrushState = BRUSH_BRUSH;
            break;
        case ID_MASK_ERASE_BUTTON:
            maskBrushState = BRUSH_ERASE;
            break;
        case ID_MASK_FILL_BUTTON:
            maskBrushState = BRUSH_FILL;
            break;
        case ID_MASK_SELECT_BUTTON:
            maskBrushState = BRUSH_SELECT;
            printl("Please click on image to select color...\n");
            break;
        case ID_MASK_DILATE_BUTTON:
            printl("Dilating mask...\n");
            cvDilate( globalInpainter[IMGI_MASK], globalInpainter[IMGI_MASK], 0, maskDilate  );
            break;
        case ID_MASK_BORDER_BUTTON:
            printl("Bordering mask...\n");
            IplImage *tempMask = cvCloneImage( globalInpainter[IMGI_MASK] );
            cvDilate( tempMask, tempMask, 0, maskBorder );
            for(int a = 0; a < globalInpainter[IMGI_MASK]->height; a++) {
                for(int b = 0; b < globalInpainter[IMGI_MASK]->width; b++) {
                    if( iPixel( globalInpainter[IMGI_MASK], a, b) > 128  ) {
                        iSixel( tempMask, a, b, 0.0 );
                    }
                }
            }
            cvCopyImage( tempMask, globalInpainter[IMGI_MASK] );
            cvReleaseImage( &tempMask );
            break;
    }
    windowUpdateResult();
}

void maskMoveEvent(int x, int y) {
    switch( maskBrushState ) {
        case BRUSH_BRUSH:
            for(int a = -maskBrushSize; a <= maskBrushSize; a++) {
                for(int b = -maskBrushSize; b <= maskBrushSize; b++) {
                    if( (a*a) + (b*b) > (maskBrushSize*maskBrushSize) ) continue;
                    int px = x + a, py = y + b;
                    if( px < 0 || px >= globalInpainter[IMGI_MASK]->height ||
                        py < 0 || py >= globalInpainter[IMGI_MASK]->width ) continue;
                    iSixel( globalInpainter[IMGI_MASK], px, py, 255.0 );
                }
            }
            windowUpdateResult();
            break;
        case BRUSH_ERASE:
            for(int a = -maskBrushSize; a <= maskBrushSize; a++) {
                for(int b = -maskBrushSize; b <= maskBrushSize; b++) {
                    if( (a*a) + (b*b) > (maskBrushSize*maskBrushSize) ) continue;
                    int px = x + a, py = y + b;
                    if( px < 0 || px >= globalInpainter[IMGI_MASK]->height ||
                        py < 0 || py >= globalInpainter[IMGI_MASK]->width ) continue;
                    iSixel( globalInpainter[IMGI_MASK], px, py, 0.0 );
                }
            }
            windowUpdateResult();
            break;
    }
}

void maskClickEvent(int x, int y) {
    if( x < 0 || x >= globalInpainter[IMGI_MASK]->height ) return;
    if( y < 0 || y >= globalInpainter[IMGI_MASK]->width ) return;
    switch( maskBrushState ) {
        case BRUSH_FILL:
            
            windowUpdateResult();
            break;
        case BRUSH_SELECT:
            CvScalar fillPix = cvGet2D( globalInpainter[IMGI_SRC], x, y );
            for(int a = 0; a < globalInpainter[IMGI_SRC]->height; a++) {
                for(int b = 0; b < globalInpainter[IMGI_SRC]->width; b++) {
                    CvScalar thisPix = cvGet2D( globalInpainter[IMGI_SRC], a, b );
                    double dis = 0.0;
                    for(int k = 0; k < globalInpainter[IMGI_SRC]->nChannels; k++) {
                        dis += iSq( fillPix.val[k] - thisPix.val[k] );
                    }
                    if( dis < MASK_SELECT_THRESHOLD ) {
                        iSixel( globalInpainter[IMGI_MASK], a, b, 255.0 );
                    } else {
                        iSixel( globalInpainter[IMGI_MASK], a, b, 0.0 );
                    }
                }
            }
            windowUpdateResult();
            break;
    }
}

void maskMouseEvent( int ev, int x, int y, int flags, void* param ) {
    if( ev == CV_EVENT_LBUTTONDOWN || ev == CV_EVENT_RBUTTONDOWN ) {
        maskMouseButtonDown = true;
        maskClickEvent( y, x );
    } 
    if( ev == CV_EVENT_LBUTTONUP || ev == CV_EVENT_RBUTTONUP ) {
        maskMouseButtonDown = false;
    }
    if( ev == CV_EVENT_MOUSEMOVE ) {
        if( maskMouseButtonDown ) maskMoveEvent( y, x );
    }
}

void printl(const char *fmt, ...) {
    char buffer[LOG_BUFFERSIZE];
    va_list ap; va_start(ap, fmt);vsprintf(buffer, fmt, ap);va_end(ap);
    logString.push_back(buffer);
    while( logString.size() > LOG_SCROLLBACK ) logString.erase( logString.begin() );
    char *uiBuffer = new char[LOG_UIBUFFERSIZE];
    if( !uiBuffer ) return;
    uiBuffer[0] = '\0';
    for(int a = 0; a < logString.size(); a++) {
        strcat( uiBuffer, logString[a].c_str() );
        strcat( uiBuffer, "\r\n" );
    }
    UISetEditText( uiBuffer, hLogEdit );
    UIEditScroll( hLogEdit, 5 );
    delete[] uiBuffer;
}